void GenericCopybits(WindowPtr theWindow, WindowPtr offscreenWindowPtr, Boolean active);
void GenericResizeControls(WindowPtr theWindow, short destOverload);
void GenericGetGrowSize(WindowPtr theWindow, Rect *sizeRect);
void GenericIdleInWindow(WindowPtr theWindow, Point mouseLoc);
Boolean GenericKeyPressedInWindow(WindowPtr theWindow, unsigned char theChar);
Boolean GenericMouseClickedInWindow(WindowPtr theWindow, Point thePoint, Boolean isEditable);
void GenericActivate(WindowPtr theWindow);
void GenericDeactivate(WindowPtr theWindow);
